#include "assemble.hh"

#include <cstring>
#include <fstream>
#include <iostream>

int main(int argc, char **argv)
{
    const char *outname = NULL;
    bool have_inname = false;
    
    assembler the_assembler;
    
    for (int i = 1; i < argc; i++)
    {
        if (strcmp(argv[i], "-o") == 0)
        {
            if (++i == argc)
            {
                fprintf(stderr, "Option -o requires an argument\n");
                return 2;
            }
            outname = argv[i];
        }
        else if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
        {
            printf("%s [-o outfile] [infile...]\n", argv[0]);
            printf("%s -h\n", argv[0]);
            printf("\n");
            printf("  Reads b assembly from infile (or standard input), assembles and links it and writes b machine code to outfile (or standard output). With -h, instead output this help.\n");
            return 0;
        }
        else
        {
            FILE *fin = fopen(argv[i], "r");
            if (!fin)
            {
                std::cerr << "Could not open '" << argv[i] << "' for reading" << std::endl;
                return 1;
            }
            if (!the_assembler.assemble(fin))
                return 1;
            fclose(fin);
            have_inname = true;
        }
    }
    
    if (!have_inname)
    {
        if (!the_assembler.assemble(stdin))
            return 1;
    }
    
    if (!the_assembler.link())
        return 1;
    
    if (outname)
    {
        std::ofstream fout(outname);
        if (!fout)
        {
            std::cerr << "Could not open '" << outname << "' for writing" << std::endl;
            return 1;
        }
        the_assembler.write_data(fout);
    }
    else
    {
        the_assembler.write_data(std::cout);
    }
    
    return 0;
}
