#include "aslexer_wrap.hh"

#include <cstdio>
#include <cstring>
#include <iostream>

int listout(FILE *fasm, int address)
{
    aslexer scanner(true);
    
    scanner.set_in(fasm);
    
    while (true)
    {
        switch (scanner.lex())
        {
            case -1:
                return -1;
                
            case 0:
                printf("\n");
                return address;
                
            case TK_INSTN:
                if (strcmp(scanner.get_text(), "LBL") == 0)
                {
                    printf("\n        %s", scanner.get_text());
                    scanner.lex();
                    printf(" %s", scanner.get_text());
                }
                else if (strcmp(scanner.get_text(), "DB") == 0)
                {
                    printf("\n%#.6o  %s", address++, scanner.get_text());
                    scanner.lex();
                    printf("      %s", scanner.get_text());
                }
                else
                {
                    printf("\n%#.6o  %s", address++, scanner.get_text());
                }
                break;
                
            case TK_VALUE:
                printf("\n%#.6o          %s", address++, scanner.get_text());
                break;
                
            case TK_COMMENT:
                printf("\n        %s", scanner.get_text());
                break;
                
            default:
                printf(" %s", scanner.get_text());
                break;
        }
    }
}

int main(int argc, char **argv)
{
    bool have_inname = false;
    int address = 0;
    
    for (int i = 1; i < argc; i++)
    {
        if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
        {
            std::cout << argv[0] << " [infile...]\n"
                      << argv[0] << " -h\n"
                      << "\n"
                      << "  Reads b assembly from infile (or standard input) and prints it together with the addresses that will be used when assembled. With -h, instead output this help." << std::endl;
            return 0;
        }
        else
        {
            FILE *fin = fopen(argv[i], "r");
            if (!fin)
            {
                std::cerr << "Could not open '" << argv[i] << "' for reading" << std::endl;
                return 1;
            }
            if ((address = listout(fin, address)) < 0)
                return 1;
            fclose(fin);
            have_inname = true;
        }
    }
    
    if (!have_inname)
    {
        if ((address = listout(stdin, address)) < 0)
            return 1;
    }
    
    return 0;
}
