#include "ast.hh"

expression::expression(expressiontype type) :
    type(type),
    ival(0),
    sval(0),
    args(0)
{
    operands[0] = 0;
    operands[1] = 0;
    operands[2] = 0;
}

expression::~expression()
{
    delete sval;
    delete operands[0];
    delete operands[1];
    delete operands[2];
    delete args;
}

expression *expression::make_funcall(expression *fun, expressions *args)
{
    expression *result = new expression(EXP_FUNCALL);
    result->operands[0] = fun;
    result->args = args;
    return result;
}

expression *expression::make_unary(int operatortype, expression *operand)
{
    expression *result = new expression(EXP_UNARY);
    result->ival = operatortype;
    result->operands[0] = operand;
    return result;
}

expression *expression::make_binary(int operatortype, expression *lhs, expression *rhs)
{
    expression *result = new expression(EXP_BINARY);
    result->ival = operatortype;
    result->operands[0] = lhs;
    result->operands[1] = rhs;
    return result;
}

expression *expression::make_assign(int operatortype, expression *lhs, expression *rhs)
{
    expression *result = new expression(operatortype == OP_NONE ? EXP_ASSIGN : EXP_COMPOUND);
    result->ival = operatortype;
    result->operands[0] = lhs;
    result->operands[1] = rhs;
    return result;
}

expression *expression::make_ternary(expression *lhs, expression *mid, expression *rhs)
{
    expression *result = new expression(EXP_TERNARY);
    result->operands[0] = lhs;
    result->operands[1] = mid;
    result->operands[2] = rhs;
    return result;
}

expression *expression::make_int(char *value)
{
    expression *result = new expression(EXP_INT);
    result->sval = value;
    return result;
}

expression *expression::make_char(char *value)
{
    expression *result = new expression(EXP_CHAR);
    result->sval = value;
    return result;
}

expression *expression::make_string(char *value)
{
    expression *result = new expression(EXP_STRING);
    result->sval = value;
    return result;
}

expression *expression::make_name(char *value)
{
    expression *result = new expression(EXP_NAME);
    result->sval = value;
    return result;
}

expression *expression::make_vector(expression *vec, expression *off)
{
    return expression::make_unary(OP_DEREF, expression::make_binary(OP_ADD, vec, off));
}
