%{
#include "parser.hh"

#include <cstring>
#include <iostream>

#define YY_USER_ACTION \
    yylloc_param->first_line = yylineno;
%}

%option header-file="lexer.hh" outfile="lexer.cc"
%option yylineno noyywrap
%option bison-bridge bison-locations reentrant

%option nodefault warn

%x comment

ALPHA [A-Za-z_]
DIGIT [0-9]
SPACE [ \r\n\t]
ALDIG {ALPHA}|{DIGIT}
ESCAP \*[0e()t*'"n]

%%

"++"                      return TK_OP_INCR;
--                        return TK_OP_DECR;
!                         return TK_OP_NOT;
ifdef(`EXTENSIONS', ~                         return TK_OP_COMP;)

;                         return *yytext;
\?                        return *yytext;
:                         return *yytext;
,                         return *yytext;
\[                        return *yytext;
\]                        return *yytext;
\(                        return *yytext;
\)                        return *yytext;
\{                        return *yytext;
\}                        return *yytext;

\|                        return TK_OP_OR;
ifdef(`EXTENSIONS', \^                        return TK_OP_XOR;)
&                         return TK_OP_AND;
==                        return TK_OP_EQ;
!=                        return TK_OP_NEQ;
\<                        return TK_OP_LT;
\<=                       return TK_OP_LEQ;
\>                        return TK_OP_GT;
\>=                       return TK_OP_GEQ;
"<<"                      return TK_OP_LS;
">>"                      return TK_OP_RS;
-                         return TK_OP_SUB;
\+                        return TK_OP_ADD;
\%                        return TK_OP_MOD;
\*                        return TK_OP_MULT;
\/                        return TK_OP_DIV;
=                         yylval->ival = OP_NONE; return TK_ASSIGNMENT;
=\|                       yylval->ival = OP_OR; return TK_ASSIGNMENT;
ifdef(`EXTENSIONS', =^                        yylval->ival = OP_XOR; return TK_ASSIGNMENT;)
=&                        yylval->ival = OP_AND; return TK_ASSIGNMENT;
===                       yylval->ival = OP_EQUAL; return TK_ASSIGNMENT;
=!=                       yylval->ival = OP_NOTEQUAL; return TK_ASSIGNMENT;
=\<                       yylval->ival = OP_LESS; return TK_ASSIGNMENT;
=\<=                      yylval->ival = OP_LESSEQUAL; return TK_ASSIGNMENT;
=\>                       yylval->ival = OP_GREATER; return TK_ASSIGNMENT;
=\>=                      yylval->ival = OP_GREATEREQUAL; return TK_ASSIGNMENT;
"=<<"                     yylval->ival = OP_LEFTSHIFT; return TK_ASSIGNMENT;
"=>>"                     yylval->ival = OP_RIGHTSHIFT; return TK_ASSIGNMENT;
=-                        yylval->ival = OP_SUBTRACT; return TK_ASSIGNMENT;
=\+                       yylval->ival = OP_ADD; return TK_ASSIGNMENT;
=%                        yylval->ival = OP_MODULO; return TK_ASSIGNMENT;
=\*                       yylval->ival = OP_MULTIPLY; return TK_ASSIGNMENT;
=\/                       yylval->ival = OP_DIVIDE; return TK_ASSIGNMENT;

auto                      return TK_KW_AUTO;
ifdef(`EXTENSIONS', break                     return TK_KW_BREAK;)
case                      return TK_KW_CASE;
else                      return TK_KW_ELSE;
extrn                     return TK_KW_EXTRN;
goto                      return TK_KW_GOTO;
if                        return TK_KW_IF;
return                    return TK_KW_RETURN;
switch                    return TK_KW_SWITCH;
while                     return TK_KW_WHILE;

{ALPHA}{ALDIG}*           yylval->sval = strdup(yytext); return TK_NAME;

{DIGIT}+                  yylval->sval = strdup(yytext); return TK_INT;
'([^'*]|{ESCAP}){0,WORD_CHARS}'    yylval->sval = strdup(yytext); return TK_CHAR;
\"([^"*]|{ESCAP})*\"      yylval->sval = strdup(yytext); return TK_STRING;

{SPACE}+                  /* Skip whitespace. */

"/*"                      BEGIN(comment);
<comment>[^*]+            ;
<comment>\*+\/            BEGIN(0);
<comment>\*+              ;
<comment><<EOF>>          std::cerr << "Unterminated comment" << std::endl; return -1;

.                         std::cerr << "Unrecognized character: " << yytext << std::endl; return -1;
